@@ -30,7 +30,7 @@ APPLET_NAME = "firewall-applet"
 DATADIR = "/usr/share/" + DAEMON_NAME
 CONFIG_GLADE_NAME = CONFIG_NAME + ".glade"
 COPYRIGHT = "(C) 2010-2017 Red Hat, Inc."
-VERSION = "2.3.1"
+VERSION = "2.4.0"
 AUTHORS = [
     "Thomas Woerner <twoerner@redhat.com>",
     "Jiri Popelka <jpopelka@redhat.com>",
@@ -104,8 +104,8 @@ COMMANDS = {
     "eb": "/bin/false",
     "eb-restore": "/bin/false",
     "ipset": "/bin/false",
-    "modprobe": "/usr/bin/modprobe",
-    "rmmod": "/usr/bin/rmmod",
+    "modprobe": "/sbin/modprobe",
+    "rmmod": "/sbin/rmmod",
 }
 
 LOG_DENIED_VALUES = ["all", "unicast", "broadcast", "multicast", "off"]
