@@ -1,25 +1,9 @@
-# -*- coding: utf-8 -*-
+# SPDX-License-Identifier: GPL-2.0-or-later
 #
 # Copyright (C) 2011-2016 Red Hat, Inc.
 #
 # Authors:
 # Thomas Woerner <twoerner@redhat.com>
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
-#
-
-__all__ = [ "FirewallConfig" ]
 
 import copy
 import os
@@ -38,36 +22,45 @@ from firewall.core.io.policy import Policy, policy_reader, policy_writer
 from firewall import errors
 from firewall.errors import FirewallError
 
-class FirewallConfig(object):
+
+class FirewallConfig:
     def __init__(self, fw):
         self._fw = fw
         self.__init_vars()
 
     def __repr__(self):
-        return '%s(%r, %r, %r, %r, %r, %r, %r, %r, %r, %r, %r, %r, %r, %r, %r)' % \
-            (self.__class__,
-             self._ipsets, self._icmptypes, self._services, self._zones,
-             self._helpers, self.policy_objects,
-             self._builtin_ipsets, self._builtin_icmptypes,
-             self._builtin_services, self._builtin_zones, self._builtin_helpers,
-             self._builtin_policy_objects,
-             self._firewalld_conf, self._policies, self._direct)
+        return "%s(%r, %r, %r, %r, %r, %r, %r, %r, %r, %r, %r, %r, %r, %r)" % (
+            self.__class__,
+            self._ipsets,
+            self._icmptypes,
+            self._services,
+            self._zones,
+            self._helpers,
+            self.policy_objects,
+            self._builtin_ipsets,
+            self._builtin_icmptypes,
+            self._builtin_services,
+            self._builtin_zones,
+            self._builtin_helpers,
+            self._builtin_policy_objects,
+            self._firewalld_conf,
+            self._direct,
+        )
 
     def __init_vars(self):
-        self._ipsets = { }
-        self._icmptypes = { }
-        self._services = { }
-        self._zones = { }
-        self._helpers = { }
-        self._policy_objects = { }
-        self._builtin_ipsets = { }
-        self._builtin_icmptypes = { }
-        self._builtin_services = { }
-        self._builtin_zones = { }
-        self._builtin_helpers = { }
-        self._builtin_policy_objects = { }
+        self._ipsets = {}
+        self._icmptypes = {}
+        self._services = {}
+        self._zones = {}
+        self._helpers = {}
+        self._policy_objects = {}
+        self._builtin_ipsets = {}
+        self._builtin_icmptypes = {}
+        self._builtin_services = {}
+        self._builtin_zones = {}
+        self._builtin_helpers = {}
+        self._builtin_policy_objects = {}
         self._firewalld_conf = None
-        self._policies = None
         self._direct = None
 
     def cleanup(self):
@@ -111,11 +104,6 @@ class FirewallConfig(object):
             del self._firewalld_conf
             self._firewalld_conf = None
 
-        if self._policies:
-            self._policies.cleanup()
-            del self._policies
-            self._policies = None
-
         if self._direct:
             self._direct.cleanup()
             del self._direct
@@ -128,12 +116,28 @@ class FirewallConfig(object):
         Returns a dict of dicts of all permanent config objects.
         """
         conf_dict = {}
-        conf_dict["ipsets"] = {ipset: self.get_ipset(ipset) for ipset in self.get_ipsets()}
-        conf_dict["helpers"] = {helper: self.get_helper(helper) for helper in self.get_helpers()}
-        conf_dict["icmptypes"] = {icmptype: self.get_icmptype(icmptype) for icmptype in self.get_icmptypes()}
-        conf_dict["services"] = {service: self.get_service(service) for service in self.get_services()}
+        conf_dict["ipsets"] = {
+            ipset: self.get_ipset(ipset) for ipset in self.get_ipsets()
+        }
+        conf_dict["helpers"] = {
+            helper: self.get_helper(helper) for helper in self.get_helpers()
+        }
+        conf_dict["icmptypes"] = {
+            icmptype: self.get_icmptype(icmptype) for icmptype in self.get_icmptypes()
+        }
+        conf_dict["services"] = {
+            service: self.get_service(service) for service in self.get_services()
+        }
         conf_dict["zones"] = {zone: self.get_zone(zone) for zone in self.get_zones()}
-        conf_dict["policies"] = {policy: self.get_policy_object(policy) for policy in self.get_policy_objects()}
+        conf_dict["policies"] = {
+            policy: self.get_policy_object(policy)
+            for policy in self.get_policy_objects()
+        }
+
+        conf_dict["conf"] = {}
+        conf_dict["conf"]["FirewallBackend"] = self._firewalld_conf.get(
+            "FirewallBackend"
+        )
 
         return conf_dict
 
@@ -149,17 +153,9 @@ class FirewallConfig(object):
         order = ["ipsets", "helpers", "icmptypes", "services", "zones", "policies"]
         for io_obj_type in order:
             io_objs = all_io_objects[io_obj_type]
-            for (name, io_obj) in io_objs.items():
+            for name, io_obj in io_objs.items():
                 io_obj.check_config_dict(io_obj.export_config_dict(), all_io_objects)
 
-    # access check
-
-    def lockdown_enabled(self):
-        return self._fw.policies.query_lockdown()
-
-    def access_check(self, key, value):
-        return self._fw.policies.access_check(key, value)
-
     # firewalld_conf
 
     def set_firewalld_conf(self, conf):
@@ -174,19 +170,28 @@ class FirewallConfig(object):
         else:
             self._firewalld_conf.read()
 
-    # policies
-
-    def set_policies(self, policies):
-        self._policies = policies
-
-    def get_policies(self):
-        return self._policies
-
-    def update_lockdown_whitelist(self):
-        if not os.path.exists(config.LOCKDOWN_WHITELIST):
-            self._policies.lockdown_whitelist.cleanup()
-        else:
-            self._policies.lockdown_whitelist.read()
+    def reset_defaults(self):
+        order = [
+            self._zones,
+            self._policy_objects,
+            self._ipsets,
+            self._services,
+            self._helpers,
+            self._icmptypes,
+        ]
+        for io_obj_dict in order:
+            dict_copy = copy.copy(io_obj_dict)
+            for obj_name in dict_copy:
+                obj = dict_copy[obj_name]
+                name = os.path.join(obj.path, obj.filename)
+                try:
+                    shutil.move(name, "%s.old" % name)
+                except Exception as msg:
+                    log.error("Backup of file '%s' failed: %s", name, msg)
+                    os.remove(name)
+                del io_obj_dict[obj_name]
+        self._firewalld_conf.set_defaults()
+        self._firewalld_conf.write()
 
     # direct
 
@@ -205,8 +210,9 @@ class FirewallConfig(object):
     # ipset
 
     def get_ipsets(self):
-        return sorted(set(list(self._ipsets.keys()) + \
-                          list(self._builtin_ipsets.keys())))
+        return sorted(
+            set(list(self._ipsets.keys()) + list(self._builtin_ipsets.keys()))
+        )
 
     def add_ipset(self, obj):
         if obj.builtin:
@@ -225,11 +231,13 @@ class FirewallConfig(object):
         if obj.name not in self._ipsets:
             raise FirewallError(errors.NO_DEFAULTS, obj.name)
         elif self._ipsets[obj.name] != obj:
-            raise FirewallError(errors.NO_DEFAULTS,
-                                "self._ipsets[%s] != obj" % obj.name)
+            raise FirewallError(
+                errors.NO_DEFAULTS, "self._ipsets[%s] != obj" % obj.name
+            )
         elif obj.name not in self._builtin_ipsets:
-            raise FirewallError(errors.NO_DEFAULTS,
-                            "'%s' not a built-in ipset" % obj.name)
+            raise FirewallError(
+                errors.NO_DEFAULTS, "'%s' not a built-in ipset" % obj.name
+            )
         self._remove_ipset(obj)
         return self._builtin_ipsets[obj.name]
 
@@ -252,8 +260,7 @@ class FirewallConfig(object):
 
     def new_ipset(self, name, conf):
         if name in self._ipsets or name in self._builtin_ipsets:
-            raise FirewallError(errors.NAME_CONFLICT,
-                                "new_ipset(): '%s'" % name)
+            raise FirewallError(errors.NAME_CONFLICT, "new_ipset(): '%s'" % name)
 
         x = IPSet()
         x.check_name(name)
@@ -343,9 +350,10 @@ class FirewallConfig(object):
         if obj.name not in self._ipsets:
             raise FirewallError(errors.INVALID_IPSET, obj.name)
         if obj.path != config.ETC_FIREWALLD_IPSETS:
-            raise FirewallError(errors.INVALID_DIRECTORY,
-                                "'%s' != '%s'" % (obj.path,
-                                                  config.ETC_FIREWALLD_IPSETS))
+            raise FirewallError(
+                errors.INVALID_DIRECTORY,
+                "'%s' != '%s'" % (obj.path, config.ETC_FIREWALLD_IPSETS),
+            )
 
         name = "%s/%s.xml" % (obj.path, obj.name)
         try:
@@ -358,8 +366,9 @@ class FirewallConfig(object):
 
     def check_builtin_ipset(self, obj):
         if obj.builtin or not obj.default:
-            raise FirewallError(errors.BUILTIN_IPSET,
-                                "'%s' is built-in ipset" % obj.name)
+            raise FirewallError(
+                errors.BUILTIN_IPSET, "'%s' is built-in ipset" % obj.name
+            )
 
     def remove_ipset(self, obj):
         self.check_builtin_ipset(obj)
@@ -377,8 +386,9 @@ class FirewallConfig(object):
     # icmptypes
 
     def get_icmptypes(self):
-        return sorted(set(list(self._icmptypes.keys()) + \
-                          list(self._builtin_icmptypes.keys())))
+        return sorted(
+            set(list(self._icmptypes.keys()) + list(self._builtin_icmptypes.keys()))
+        )
 
     def add_icmptype(self, obj):
         if obj.builtin:
@@ -397,11 +407,13 @@ class FirewallConfig(object):
         if obj.name not in self._icmptypes:
             raise FirewallError(errors.NO_DEFAULTS, obj.name)
         elif self._icmptypes[obj.name] != obj:
-            raise FirewallError(errors.NO_DEFAULTS,
-                                "self._icmptypes[%s] != obj" % obj.name)
+            raise FirewallError(
+                errors.NO_DEFAULTS, "self._icmptypes[%s] != obj" % obj.name
+            )
         elif obj.name not in self._builtin_icmptypes:
-            raise FirewallError(errors.NO_DEFAULTS,
-                                "'%s' not a built-in icmptype" % obj.name)
+            raise FirewallError(
+                errors.NO_DEFAULTS, "'%s' not a built-in icmptype" % obj.name
+            )
         self._remove_icmptype(obj)
         return self._builtin_icmptypes[obj.name]
 
@@ -424,8 +436,7 @@ class FirewallConfig(object):
 
     def new_icmptype(self, name, conf):
         if name in self._icmptypes or name in self._builtin_icmptypes:
-            raise FirewallError(errors.NAME_CONFLICT,
-                                "new_icmptype(): '%s'" % name)
+            raise FirewallError(errors.NAME_CONFLICT, "new_icmptype(): '%s'" % name)
 
         x = IcmpType()
         x.check_name(name)
@@ -515,9 +526,10 @@ class FirewallConfig(object):
         if obj.name not in self._icmptypes:
             raise FirewallError(errors.INVALID_ICMPTYPE, obj.name)
         if obj.path != config.ETC_FIREWALLD_ICMPTYPES:
-            raise FirewallError(errors.INVALID_DIRECTORY,
-                                "'%s' != '%s'" % \
-                                (obj.path, config.ETC_FIREWALLD_ICMPTYPES))
+            raise FirewallError(
+                errors.INVALID_DIRECTORY,
+                "'%s' != '%s'" % (obj.path, config.ETC_FIREWALLD_ICMPTYPES),
+            )
 
         name = "%s/%s.xml" % (obj.path, obj.name)
         try:
@@ -530,8 +542,9 @@ class FirewallConfig(object):
 
     def check_builtin_icmptype(self, obj):
         if obj.builtin or not obj.default:
-            raise FirewallError(errors.BUILTIN_ICMPTYPE,
-                                "'%s' is built-in icmp type" % obj.name)
+            raise FirewallError(
+                errors.BUILTIN_ICMPTYPE, "'%s' is built-in icmp type" % obj.name
+            )
 
     def remove_icmptype(self, obj):
         self.check_builtin_icmptype(obj)
@@ -549,8 +562,9 @@ class FirewallConfig(object):
     # services
 
     def get_services(self):
-        return sorted(set(list(self._services.keys()) + \
-                          list(self._builtin_services.keys())))
+        return sorted(
+            set(list(self._services.keys()) + list(self._builtin_services.keys()))
+        )
 
     def add_service(self, obj):
         if obj.builtin:
@@ -569,34 +583,24 @@ class FirewallConfig(object):
         if obj.name not in self._services:
             raise FirewallError(errors.NO_DEFAULTS, obj.name)
         elif self._services[obj.name] != obj:
-            raise FirewallError(errors.NO_DEFAULTS,
-                                "self._services[%s] != obj" % obj.name)
+            raise FirewallError(
+                errors.NO_DEFAULTS, "self._services[%s] != obj" % obj.name
+            )
         elif obj.name not in self._builtin_services:
-            raise FirewallError(errors.NO_DEFAULTS,
-                                "'%s' not a built-in service" % obj.name)
+            raise FirewallError(
+                errors.NO_DEFAULTS, "'%s' not a built-in service" % obj.name
+            )
         self._remove_service(obj)
         return self._builtin_services[obj.name]
 
     def get_service_config(self, obj):
-        conf_dict = obj.export_config_dict()
-        conf_list = []
-        for i in range(8): # tuple based dbus API has 8 elements
-            if obj.IMPORT_EXPORT_STRUCTURE[i][0] not in conf_dict:
-                # old API needs the empty elements as well. Grab it from the
-                # object otherwise we don't know the type.
-                conf_list.append(copy.deepcopy(getattr(obj, obj.IMPORT_EXPORT_STRUCTURE[i][0])))
-            else:
-                conf_list.append(conf_dict[obj.IMPORT_EXPORT_STRUCTURE[i][0]])
-        return tuple(conf_list)
+        return obj.export_config_tuple(length=8)
 
     def get_service_config_dict(self, obj):
         return obj.export_config_dict()
 
     def set_service_config(self, obj, conf):
-        conf_dict = {}
-        for i,value in enumerate(conf):
-            conf_dict[obj.IMPORT_EXPORT_STRUCTURE[i][0]] = value
-
+        conf_dict = obj.get_dict_from_tuple(conf)
         return self.set_service_config_dict(obj, conf_dict)
 
     def set_service_config_dict(self, obj, conf):
@@ -615,19 +619,16 @@ class FirewallConfig(object):
 
     def new_service(self, name, conf):
         if name in self._services or name in self._builtin_services:
-            raise FirewallError(errors.NAME_CONFLICT,
-                                "new_service(): '%s'" % name)
-
-        conf_dict = {}
-        for i,value in enumerate(conf):
-            conf_dict[Service.IMPORT_EXPORT_STRUCTURE[i][0]] = value
+            raise FirewallError(errors.NAME_CONFLICT, "new_service(): '%s'" % name)
 
+        conf_dict = IO_Object.get_dict_from_tuple_static(
+            Service.IMPORT_EXPORT_STRUCTURE, conf
+        )
         return self.new_service_dict(name, conf_dict)
 
     def new_service_dict(self, name, conf):
         if name in self._services or name in self._builtin_services:
-            raise FirewallError(errors.NAME_CONFLICT,
-                                "new_service(): '%s'" % name)
+            raise FirewallError(errors.NAME_CONFLICT, "new_service(): '%s'" % name)
 
         x = Service()
         x.check_name(name)
@@ -717,9 +718,10 @@ class FirewallConfig(object):
         if obj.name not in self._services:
             raise FirewallError(errors.INVALID_SERVICE, obj.name)
         if obj.path != config.ETC_FIREWALLD_SERVICES:
-            raise FirewallError(errors.INVALID_DIRECTORY,
-                                "'%s' != '%s'" % \
-                                (obj.path, config.ETC_FIREWALLD_SERVICES))
+            raise FirewallError(
+                errors.INVALID_DIRECTORY,
+                "'%s' != '%s'" % (obj.path, config.ETC_FIREWALLD_SERVICES),
+            )
 
         name = "%s/%s.xml" % (obj.path, obj.name)
         try:
@@ -732,8 +734,9 @@ class FirewallConfig(object):
 
     def check_builtin_service(self, obj):
         if obj.builtin or not obj.default:
-            raise FirewallError(errors.BUILTIN_SERVICE,
-                                "'%s' is built-in service" % obj.name)
+            raise FirewallError(
+                errors.BUILTIN_SERVICE, "'%s' is built-in service" % obj.name
+            )
 
     def remove_service(self, obj):
         self.check_builtin_service(obj)
@@ -751,8 +754,7 @@ class FirewallConfig(object):
     # zones
 
     def get_zones(self):
-        return sorted(set(list(self._zones.keys()) + \
-                          list(self._builtin_zones.keys())))
+        return sorted(set(list(self._zones.keys()) + list(self._builtin_zones.keys())))
 
     def add_zone(self, obj):
         if obj.builtin:
@@ -777,34 +779,22 @@ class FirewallConfig(object):
         if obj.name not in self._zones:
             raise FirewallError(errors.NO_DEFAULTS, obj.name)
         elif self._zones[obj.name] != obj:
-            raise FirewallError(errors.NO_DEFAULTS,
-                                "self._zones[%s] != obj" % obj.name)
+            raise FirewallError(errors.NO_DEFAULTS, "self._zones[%s] != obj" % obj.name)
         elif obj.name not in self._builtin_zones:
-            raise FirewallError(errors.NO_DEFAULTS,
-                                "'%s' not a built-in zone" % obj.name)
+            raise FirewallError(
+                errors.NO_DEFAULTS, "'%s' not a built-in zone" % obj.name
+            )
         self._remove_zone(obj)
         return self._builtin_zones[obj.name]
 
     def get_zone_config(self, obj):
-        conf_dict = obj.export_config_dict()
-        conf_list = []
-        for i in range(16): # tuple based dbus API has 16 elements
-            if obj.IMPORT_EXPORT_STRUCTURE[i][0] not in conf_dict:
-                # old API needs the empty elements as well. Grab it from the
-                # object otherwise we don't know the type.
-                conf_list.append(copy.deepcopy(getattr(obj, obj.IMPORT_EXPORT_STRUCTURE[i][0])))
-            else:
-                conf_list.append(conf_dict[obj.IMPORT_EXPORT_STRUCTURE[i][0]])
-        return tuple(conf_list)
+        return obj.export_config_tuple(length=16)
 
     def get_zone_config_dict(self, obj):
         return obj.export_config_dict()
 
     def set_zone_config(self, obj, conf):
-        conf_dict = {}
-        for i,value in enumerate(conf):
-            conf_dict[obj.IMPORT_EXPORT_STRUCTURE[i][0]] = value
-
+        conf_dict = obj.get_dict_from_tuple(conf)
         return self.set_zone_config_dict(obj, conf_dict)
 
     def set_zone_config_dict(self, obj, conf):
@@ -825,10 +815,9 @@ class FirewallConfig(object):
         if name in self._zones or name in self._builtin_zones:
             raise FirewallError(errors.NAME_CONFLICT, "new_zone(): '%s'" % name)
 
-        conf_dict = {}
-        for i,value in enumerate(conf):
-            conf_dict[Zone.IMPORT_EXPORT_STRUCTURE[i][0]] = value
-
+        conf_dict = IO_Object.get_dict_from_tuple_static(
+            Zone.IMPORT_EXPORT_STRUCTURE, conf
+        )
         return self.new_zone_dict(name, conf_dict)
 
     def new_zone_dict(self, name, conf):
@@ -891,11 +880,14 @@ class FirewallConfig(object):
             log.error("Failed to load zone file '%s': %s", filename, msg)
             return (None, None)
 
-        if path.startswith(config.ETC_FIREWALLD_ZONES) and \
-           len(path) > len(config.ETC_FIREWALLD_ZONES):
+        if path.startswith(config.ETC_FIREWALLD_ZONES) and len(path) > len(
+            config.ETC_FIREWALLD_ZONES
+        ):
             # custom combined zone part
-            obj.name = "%s/%s" % (os.path.basename(path),
-                                  os.path.basename(filename)[0:-4])
+            obj.name = "%s/%s" % (
+                os.path.basename(path),
+                os.path.basename(filename)[0:-4],
+            )
 
         # new zone
         if obj.name not in self._builtin_zones and obj.name not in self._zones:
@@ -929,9 +921,10 @@ class FirewallConfig(object):
         if obj.name not in self._zones:
             raise FirewallError(errors.INVALID_ZONE, obj.name)
         if not obj.path.startswith(config.ETC_FIREWALLD_ZONES):
-            raise FirewallError(errors.INVALID_DIRECTORY,
-                                "'%s' doesn't start with '%s'" % \
-                                (obj.path, config.ETC_FIREWALLD_ZONES))
+            raise FirewallError(
+                errors.INVALID_DIRECTORY,
+                "'%s' doesn't start with '%s'" % (obj.path, config.ETC_FIREWALLD_ZONES),
+            )
 
         name = "%s/%s.xml" % (obj.path, obj.name)
         try:
@@ -944,8 +937,7 @@ class FirewallConfig(object):
 
     def check_builtin_zone(self, obj):
         if obj.builtin or not obj.default:
-            raise FirewallError(errors.BUILTIN_ZONE,
-                                "'%s' is built-in zone" % obj.name)
+            raise FirewallError(errors.BUILTIN_ZONE, "'%s' is built-in zone" % obj.name)
 
     def remove_zone(self, obj):
         self.check_builtin_zone(obj)
@@ -966,8 +958,12 @@ class FirewallConfig(object):
     # policy objects
 
     def get_policy_objects(self):
-        return sorted(set(list(self._policy_objects.keys()) + \
-                          list(self._builtin_policy_objects.keys())))
+        return sorted(
+            set(
+                list(self._policy_objects.keys())
+                + list(self._builtin_policy_objects.keys())
+            )
+        )
 
     def add_policy_object(self, obj):
         if obj.builtin:
@@ -986,11 +982,13 @@ class FirewallConfig(object):
         if obj.name not in self._policy_objects:
             raise FirewallError(errors.NO_DEFAULTS, obj.name)
         elif self._policy_objects[obj.name] != obj:
-            raise FirewallError(errors.NO_DEFAULTS,
-                                "self._policy_objects[%s] != obj" % obj.name)
+            raise FirewallError(
+                errors.NO_DEFAULTS, "self._policy_objects[%s] != obj" % obj.name
+            )
         elif obj.name not in self._builtin_policy_objects:
-            raise FirewallError(errors.NO_DEFAULTS,
-                                "'%s' not a built-in policy" % obj.name)
+            raise FirewallError(
+                errors.NO_DEFAULTS, "'%s' not a built-in policy" % obj.name
+            )
         self._remove_policy_object(obj)
         return self._builtin_policy_objects[obj.name]
 
@@ -1013,7 +1011,9 @@ class FirewallConfig(object):
 
     def new_policy_object_dict(self, name, conf):
         if name in self._policy_objects or name in self._builtin_policy_objects:
-            raise FirewallError(errors.NAME_CONFLICT, "new_policy_object(): '%s'" % name)
+            raise FirewallError(
+                errors.NAME_CONFLICT, "new_policy_object(): '%s'" % name
+            )
 
         x = Policy()
         x.check_name(name)
@@ -1071,14 +1071,20 @@ class FirewallConfig(object):
             log.error("Failed to load policy file '%s': %s", filename, msg)
             return (None, None)
 
-        if path.startswith(config.ETC_FIREWALLD_POLICIES) and \
-           len(path) > len(config.ETC_FIREWALLD_POLICIES):
+        if path.startswith(config.ETC_FIREWALLD_POLICIES) and len(path) > len(
+            config.ETC_FIREWALLD_POLICIES
+        ):
             # custom combined policy_object part
-            obj.name = "%s/%s" % (os.path.basename(path),
-                                  os.path.basename(filename)[0:-4])
+            obj.name = "%s/%s" % (
+                os.path.basename(path),
+                os.path.basename(filename)[0:-4],
+            )
 
         # new policy_object
-        if obj.name not in self._builtin_policy_objects and obj.name not in self._policy_objects:
+        if (
+            obj.name not in self._builtin_policy_objects
+            and obj.name not in self._policy_objects
+        ):
             self.add_policy_object(obj)
             return ("new", obj)
 
@@ -1109,9 +1115,11 @@ class FirewallConfig(object):
         if obj.name not in self._policy_objects:
             raise FirewallError(errors.INVALID_POLICY, obj.name)
         if not obj.path.startswith(config.ETC_FIREWALLD_POLICIES):
-            raise FirewallError(errors.INVALID_DIRECTORY,
-                                "'%s' doesn't start with '%s'" % \
-                                (obj.path, config.ETC_FIREWALLD_POLICIES))
+            raise FirewallError(
+                errors.INVALID_DIRECTORY,
+                "'%s' doesn't start with '%s'"
+                % (obj.path, config.ETC_FIREWALLD_POLICIES),
+            )
 
         name = "%s/%s.xml" % (obj.path, obj.name)
         try:
@@ -1124,8 +1132,9 @@ class FirewallConfig(object):
 
     def check_builtin_policy_object(self, obj):
         if obj.builtin or not obj.default:
-            raise FirewallError(errors.BUILTIN_POLICY,
-                                "'%s' is built-in policy" % obj.name)
+            raise FirewallError(
+                errors.BUILTIN_POLICY, "'%s' is built-in policy" % obj.name
+            )
 
     def remove_policy_object(self, obj):
         self.check_builtin_policy_object(obj)
@@ -1143,8 +1152,9 @@ class FirewallConfig(object):
     # helper
 
     def get_helpers(self):
-        return sorted(set(list(self._helpers.keys()) + \
-                          list(self._builtin_helpers.keys())))
+        return sorted(
+            set(list(self._helpers.keys()) + list(self._builtin_helpers.keys()))
+        )
 
     def add_helper(self, obj):
         if obj.builtin:
@@ -1163,11 +1173,13 @@ class FirewallConfig(object):
         if obj.name not in self._helpers:
             raise FirewallError(errors.NO_DEFAULTS, obj.name)
         elif self._helpers[obj.name] != obj:
-            raise FirewallError(errors.NO_DEFAULTS,
-                                "self._helpers[%s] != obj" % obj.name)
+            raise FirewallError(
+                errors.NO_DEFAULTS, "self._helpers[%s] != obj" % obj.name
+            )
         elif obj.name not in self._builtin_helpers:
-            raise FirewallError(errors.NO_DEFAULTS,
-                            "'%s' not a built-in helper" % obj.name)
+            raise FirewallError(
+                errors.NO_DEFAULTS, "'%s' not a built-in helper" % obj.name
+            )
         self._remove_helper(obj)
         return self._builtin_helpers[obj.name]
 
@@ -1190,8 +1202,7 @@ class FirewallConfig(object):
 
     def new_helper(self, name, conf):
         if name in self._helpers or name in self._builtin_helpers:
-            raise FirewallError(errors.NAME_CONFLICT,
-                                "new_helper(): '%s'" % name)
+            raise FirewallError(errors.NAME_CONFLICT, "new_helper(): '%s'" % name)
 
         x = Helper()
         x.check_name(name)
@@ -1281,9 +1292,10 @@ class FirewallConfig(object):
         if obj.name not in self._helpers:
             raise FirewallError(errors.INVALID_HELPER, obj.name)
         if obj.path != config.ETC_FIREWALLD_HELPERS:
-            raise FirewallError(errors.INVALID_DIRECTORY,
-                                "'%s' != '%s'" % (obj.path,
-                                                  config.ETC_FIREWALLD_HELPERS))
+            raise FirewallError(
+                errors.INVALID_DIRECTORY,
+                "'%s' != '%s'" % (obj.path, config.ETC_FIREWALLD_HELPERS),
+            )
 
         name = "%s/%s.xml" % (obj.path, obj.name)
         try:
@@ -1296,8 +1308,9 @@ class FirewallConfig(object):
 
     def check_builtin_helper(self, obj):
         if obj.builtin or not obj.default:
-            raise FirewallError(errors.BUILTIN_HELPER,
-                                "'%s' is built-in helper" % obj.name)
+            raise FirewallError(
+                errors.BUILTIN_HELPER, "'%s' is built-in helper" % obj.name
+            )
 
     def remove_helper(self, obj):
         self.check_builtin_helper(obj)
